#!/bin/bash
#
# Filesystem Checksum Tool (fschksum) - minimal tool to generate files that can
# be imported into a DB by fschksum
#
# Copyright 2021,2022 Christian Schneider <software(at)chschneider(dot)eu>
#
# This file is part of the Filesystem Checksum Tool (fschksum)
# (called "program" in the following).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation, *not* any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

## strip trailing backslashes '/', if any
ARG="${1}"
REGEX='/$'
while [[ ${#ARG} -gt 1 && ${ARG} =~ ${REGEX} ]]; do
  ARG="${ARG:0:-1}"
done

if [[ ${#} != "1" ]]; then
  echo "Usage: ${0} <basedir>" >&2
  exit 1
fi
if [[ -e "${ARG}.FILES" ]]; then
  echo "ERROR: \`${ARG}.FILES' already exists." >&2
  exit 1
fi
if [[ -e "${ARG}.MD5SUMS" ]]; then
  echo "ERROR: \`${ARG}.MD5SUMS' already exists." >&2
  exit 1
fi
if [[ ! -d "${ARG}" ]]; then
  echo "ERROR: \`${ARG}' is not a directory." >&2
  exit 1
fi

(
  cd "${ARG}" &&
  find . -xdev \( -type f -o -type d \) -printf '%i %Ts %s %y %p\n'
) > "${ARG}.FILES"


sort -n "${ARG}.FILES" | gawk -vDIR="${ARG}" '
  {
    if (($1 != INO_PREV || $2 != MTIME_PREV) && $4 == "f") {
      INO   = $1
      MTIME = $2
      $1 = $2 = $3 = $4 = ""
      FILE = gensub(/^    /, "", "1", $0)
      CMD = "md5sum \"" DIR "/" FILE "\""
      CMD | getline M
      close(CMD)
      print INO " " MTIME " " gensub(/ .*/, "", "g", M)
    }
    INO_PREV   = INO
    MTIME_PREV = MTIME
  }
' > "${ARG}.MD5SUMS"
